@@ -12,7 +12,9 @@ import os
 import hmac
 import hashlib
 import time
+from flask import Blueprint
 from flask import current_app, session, request, abort
+from werkzeug.security import safe_str_cmp
 from ._compat import to_bytes, string_types
 try:
     from urlparse import urlparse
@@ -47,7 +49,7 @@ def generate_csrf(secret_key=None, time_limit=None):
         session['csrf_token'] = hashlib.sha1(os.urandom(64)).hexdigest()
 
     if time_limit:
-        expires = time.time() + time_limit
+        expires = int(time.time() + time_limit)
         csrf_build = '%s%s' % (session['csrf_token'], expires)
     else:
         expires = ''
@@ -73,17 +75,21 @@ def validate_csrf(data, secret_key=None, time_limit=None):
     if not data or '##' not in data:
         return False
 
-    expires, hmac_csrf = data.split('##', 1)
     try:
-        expires = float(expires)
-    except:
-        return False
+        expires, hmac_csrf = data.split('##', 1)
+    except ValueError:
+        return False  # unpack error
 
     if time_limit is None:
         time_limit = current_app.config.get('WTF_CSRF_TIME_LIMIT', 3600)
 
     if time_limit:
-        now = time.time()
+        try:
+            expires = int(expires)
+        except ValueError:
+            return False
+
+        now = int(time.time())
         if now > expires:
             return False
 
@@ -101,7 +107,8 @@ def validate_csrf(data, secret_key=None, time_limit=None):
         to_bytes(csrf_build),
         digestmod=hashlib.sha1
     ).hexdigest()
-    return hmac_compare == hmac_csrf
+
+    return safe_str_cmp(hmac_compare, hmac_csrf)
 
 
 class CsrfProtect(object):
@@ -125,13 +132,35 @@ class CsrfProtect(object):
 
     def __init__(self, app=None):
         self._exempt_views = set()
+        self._exempt_blueprints = set()
 
         if app:
             self.init_app(app)
 
     def init_app(self, app):
+        app.jinja_env.globals['csrf_token'] = generate_csrf
+        app.config.setdefault(
+            'WTF_CSRF_HEADERS', ['X-CSRFToken', 'X-CSRF-Token']
+        )
         app.config.setdefault('WTF_CSRF_SSL_STRICT', True)
         app.config.setdefault('WTF_CSRF_ENABLED', True)
+        app.config.setdefault('WTF_CSRF_METHODS', ['POST', 'PUT', 'PATCH'])
+
+        def _get_csrf_token():
+            # find the ``csrf_token`` field in the subitted form
+            # if the form had a prefix, the name will be
+            # ``{prefix}-csrf_token``
+            for key in request.form:
+                if key.endswith('csrf_token'):
+                    csrf_token = request.form[key]
+                    if csrf_token:
+                        return csrf_token
+
+            for header_name in app.config['WTF_CSRF_HEADERS']:
+                csrf_token = request.headers.get(header_name)
+                if csrf_token:
+                    return csrf_token
+            return None
 
         # expose csrf_token as a helper in all templates
         @app.context_processor
@@ -144,10 +173,10 @@ class CsrfProtect(object):
             if not app.config['WTF_CSRF_ENABLED']:
                 return
 
-            if request.method in ('GET', 'HEAD', 'OPTIONS', 'TRACE'):
+            if request.method not in app.config['WTF_CSRF_METHODS']:
                 return
 
-            if self._exempt_views:
+            if self._exempt_views or self._exempt_blueprints:
                 if not request.endpoint:
                     return
 
@@ -158,22 +187,10 @@ class CsrfProtect(object):
                 dest = '%s.%s' % (view.__module__, view.__name__)
                 if dest in self._exempt_views:
                     return
+                if request.blueprint in self._exempt_blueprints:
+                    return
 
-            csrf_token = None
-            if request.method in ('POST', 'PUT', 'PATCH'):
-                # find the ``csrf_token`` field in the subitted form
-                # if the form had a prefix, the name will be ``{prefix}-csrf_token``
-                for key in request.form:
-                    if key.endswith('csrf_token'):
-                        csrf_token = request.form[key]
-            if not csrf_token:
-                # You can get csrf token from header
-                # The header name is the same as Django
-                csrf_token = request.headers.get('X-CSRFToken')
-            if not csrf_token:
-                # The header name is the same as Rails
-                csrf_token = request.headers.get('X-CSRF-Token')
-            if not validate_csrf(csrf_token):
+            if not validate_csrf(_get_csrf_token()):
                 reason = 'CSRF token missing or incorrect.'
                 return self._error_response(reason)
 
@@ -201,6 +218,9 @@ class CsrfProtect(object):
             def some_view():
                 return
         """
+        if isinstance(view, Blueprint):
+            self._exempt_blueprints.add(view.name)
+            return view
         if isinstance(view, string_types):
             view_location = view
         else:
