@@ -1,6 +1,7 @@
 from __future__ import with_statement
 
 import re
+from flask import Blueprint
 from flask import render_template
 from flask_wtf.csrf import CsrfProtect
 from flask_wtf.csrf import validate_csrf, generate_csrf
@@ -37,6 +38,14 @@ class TestCSRF(TestCase):
                 "index.html", form=form, name=name
             )
 
+        bp = Blueprint('csrf', __name__)
+
+        @bp.route('/foo', methods=['GET', 'POST'])
+        def foo():
+            return 'foo'
+
+        app.register_blueprint(bp, url_prefix='/bar')
+        self.bp = bp
         self.app = app
         self.client = self.app.test_client()
 
@@ -161,6 +170,40 @@ class TestCSRF(TestCase):
         )
         assert response.status_code == 200
 
+    def test_empty_csrf_headers(self):
+        response = self.client.get("/", base_url='https://localhost/')
+        csrf_token = get_csrf_token(response.data)
+        self.app.config['WTF_CSRF_HEADERS'] = list()
+        response = self.client.post(
+            "/",
+            data={"name": "danny"},
+            headers={
+                'X-CSRFToken': csrf_token,
+            },
+            environ_base={
+                'HTTP_REFERER': 'https://localhost/',
+            },
+            base_url='https://localhost/',
+        )
+        assert response.status_code == 400
+
+    def test_custom_csrf_headers(self):
+        response = self.client.get("/", base_url='https://localhost/')
+        csrf_token = get_csrf_token(response.data)
+        self.app.config['WTF_CSRF_HEADERS'] = ['X-XSRF-TOKEN']
+        response = self.client.post(
+            "/",
+            data={"name": "danny"},
+            headers={
+                'X-XSRF-TOKEN': csrf_token,
+            },
+            environ_base={
+                'HTTP_REFERER': 'https://localhost/',
+            },
+            base_url='https://localhost/',
+        )
+        assert response.status_code == 200
+
     def test_not_endpoint(self):
         response = self.client.post('/not-endpoint')
         assert response.status_code == 404
@@ -185,6 +228,11 @@ class TestCSRF(TestCase):
             csrf_token = generate_csrf()
             assert validate_csrf(csrf_token)
 
+    def test_validate_not_expiring_csrf(self):
+        with self.app.test_request_context():
+            csrf_token = generate_csrf(time_limit=False)
+            assert validate_csrf(csrf_token, time_limit=False)
+
     def test_csrf_token_helper(self):
         @self.app.route("/token")
         def withtoken():
@@ -192,3 +240,19 @@ class TestCSRF(TestCase):
 
         response = self.client.get('/token')
         assert b'#' in response.data
+
+    def test_csrf_blueprint(self):
+        response = self.client.post('/bar/foo')
+        assert response.status_code == 400
+
+        self.csrf.exempt(self.bp)
+        response = self.client.post('/bar/foo')
+        assert response.status_code == 200
+
+    def test_csrf_token_macro(self):
+        @self.app.route("/token")
+        def withtoken():
+            return render_template("import_csrf.html")
+
+        response = self.client.get('/token')
+        assert b'#' in response.data
